'use strict'
var __importDefault =
	(this && this.__importDefault) ||
	function (mod) {
		return mod && mod.__esModule ? mod : { default: mod }
	}
Object.defineProperty(exports, '__esModule', { value: true })
exports.registry = exports.remote = exports.file = exports.json = void 0
const cross_fetch_1 = __importDefault(require('cross-fetch'))
const errlop_1 = __importDefault(require('errlop'))
/** Trim the trailing slash off the path */
function trim(path) {
	return path.replace(/\/+$/, '')
}
/**
 * Verify that the package has a `module` field present
 * @rejects if otherwise
 */
function json(data) {
	if (!data.module) return Promise.reject(new Error('module field was missing'))
	return Promise.resolve(true)
}
exports.json = json
/**
 * Verify that the provided file contents is a valid ECMAScript Module
 * @param file the file contents that we wish to validate
 * @rejects if otherwise
 */
function file(content) {
	// search the start of newlines, and trim any indentation
	// this avoids comments and strings, except for template literals, but that is rare
	content = '\n' + content.replace(/^[ \t]+/, '') + '\n'
	if (content.includes('\nmodule.exports ='))
		return Promise.reject(new Error('module file uses [module.exports = ]'))
	if (content.includes('\nexports = '))
		return Promise.reject(new Error('module file uses [exports =]'))
	if (content.includes('\nexports.'))
		return Promise.reject(new Error('module file uses [exports.* =]'))
	if (content.includes('\nrequire('))
		return Promise.reject(
			new Error('module file used the CommonJS require method')
		)
	if (!content.includes('\nexport '))
		return Promise.reject(new Error('module file did not make use of export'))
	return Promise.resolve(true)
}
exports.file = file
/**
 * Verify that the package URL declares a valid ECMAScript module file
 * @param packageRootURL the root CDN URL of the package, e.g. `https://unpkg.com/daet@1.11.0`
 * @rejects if otherwise
 */
async function remote(packageRootURL) {
	let path = packageRootURL
	try {
		const pkgURL = trim(packageRootURL) + '/package.json'
		const pkgResponse = await cross_fetch_1.default(pkgURL)
		const data = await pkgResponse.json()
		await json(data)
		path = trim(packageRootURL) + '/' + data.module
		const fileResponse = await cross_fetch_1.default(path)
		const content = await fileResponse.text()
		return await file(content)
	} catch (err) {
		return Promise.reject(
			new errlop_1.default(`${path} is not a valid module`, err)
		)
	}
}
exports.remote = remote
/**
 * Verify that the published package declares a valid ECMAScript module file
 * Uses unpkg behind the scenes.
 * @param packageName the name of the published package that we should validate, e.g. `daet`
 * @param packageVersion the version of the published package that we should validate, e.g. `1.11.0`
 * @rejects if otherwise
 */
function registry(packageName, packageVersion) {
	const url =
		`https://unpkg.com/${packageName}` +
		(packageVersion ? `@${packageVersion}` : '')
	return remote(url)
}
exports.registry = registry
